{% extends "appbuilder/base.html" %}
{% block content %}
    <div class="alert alert-danger" role="alert">
        <strong>Error loading yaml {{yaml}}</strong>
        <p>Please check the webserver logs for more information.</p>
    </div>
{% endblock %}