{% extends 'appbuilder/baselayout.html' %}

{% block head_css %}
  {{ super() }}
  <link rel="stylesheet" type="text/css" href="static_airflow/airflowDefaultTheme.css">
  <link rel="stylesheet" type="text/css" href="static_airflow/materialIcons.css">
  <link rel="stylesheet" type="text/css" href="static_airflow/main.css">
  <!-- <link rel="stylesheet" type="text/css" href="loadingDots.css"> -->
  <!-- <link rel="stylesheet" type="text/css" href="bootstrap-datetimepicker.min.css"> -->
{% endblock %}

{% block tail_js %}
  {{ super() }}
  <!-- <script src="moment.js"></script> -->
  <!-- <script src="main.js"></script> -->
  <!-- <script src="bootstrap-datetimepicker.min.js"></script> -->
  <!-- <script src="bootstrap3-typeahead.min.js"></script> -->
  <!-- <script src="toggleTheme.js"></script> -->
  <!-- <script src="moment.js"></script> -->
{% endblock %}
